package com.link.core.converter;

import com.link.core.converter.i18n.DefaultStringValue;
import com.link.core.converter.i18n.Resources;
import com.link.core.converter.util.Assert;
import com.link.core.converter.util.RegularCloneable;
import java.lang.reflect.Array;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ArrayConverter<S, T> implements Converter<Object, T[]>, HasConverters, RegularCloneable {
   private static final boolean DEFAULT_OPTIMAL = true;
   private Converter<S, T> elementConverter;
   private T[] array;
   private boolean optimal;

   public static <E> ArrayConverter<E, E> newConverter(Class<E> elementType, E[] array) throws IllegalArgumentException {
      return new ArrayConverter(array);
   }

   public static <S, T> ArrayConverter<S, T> newConverter(Class<S> sourceElementType, Class<T> targetElementType, T[] array) throws IllegalArgumentException {
      return new ArrayConverter(array);
   }

   public static <S, T> ArrayConverter<S, T> newConverter(Converter<S, T> elementConverter, T[] array) throws IllegalArgumentException {
      return new ArrayConverter(elementConverter, array);
   }

   protected ArrayConverter(T[] array) throws IllegalArgumentException {
      this((Converter)null, array);
   }

   protected ArrayConverter(Converter<S, T> elementConverter, T[] array) throws IllegalArgumentException {
      this.optimal = true;
      Assert.assertArgumentNotNull(array, "array");
      this.elementConverter = elementConverter;
      this.array = array;
   }

   public Converter<S, T> getElementConverter() {
      return this.elementConverter;
   }

   public ArrayConverter<S, T> setElementConverter(Converter<S, T> elementConverter) {
      this.elementConverter = elementConverter;
      return this;
   }

   public boolean isOptimal() {
      return this.optimal;
   }

   public ArrayConverter<S, T> setOptimal(boolean optimal) {
      this.optimal = optimal;
      return this;
   }

   public List<Converter> getConverters() {
      List<Converter> list = new ArrayList();
      if (this.elementConverter != null) {
         list.add(this.elementConverter);
      }

      return list;
   }

   public T[] convert(Object source) throws IllegalArgumentException, ConversionException {
      if (source == null) {
         return null;
      } else {
         try {
            if (source instanceof Object[]) {
               return this.convertFromArray((Object[])((Object[])source));
            } else if (source instanceof Iterable) {
               return this.convertFromIterable((Iterable)source);
            } else {
               throw new IllegalArgumentException(MessageFormat.format(ArrayConverter.R.R.unsupportedSourceClass(), source.getClass().getName()));
            }
         } catch (ClassCastException var3) {
            ClassCastException e = var3;
            throw new IllegalArgumentException(MessageFormat.format(ArrayConverter.R.R.unsupportedSourceClass(), source.getClass().getName()), e);
         }
      }
   }

   public Object clone() {
      ArrayConverter<S, T> other = new ArrayConverter(this.array);
      other.copyFrom(this);
      return other;
   }

   public void copyFrom(Object source) {
      if (source != null && source instanceof ArrayConverter) {
         ArrayConverter<S, T> sc = (ArrayConverter)source;
         if (sc.elementConverter instanceof RegularCloneable) {
            this.elementConverter = (Converter)((RegularCloneable)sc.elementConverter).clone();
         } else {
            this.elementConverter = sc.elementConverter;
         }

         this.optimal = sc.optimal;
      }
   }

   private T[] convertFromArray(S[] source) throws ConversionException {
      assert source != null;

      T[] target = (Object[])((Object[])Array.newInstance(this.array.getClass().getComponentType(), source.length));
      Map<S, T> buffer = new HashMap();

      for(int index = 0; index < source.length; ++index) {
         S sourceElement = source[index];
         T targetElement = this.convertElement(sourceElement, buffer);
         target[index] = targetElement;
      }

      return target;
   }

   private T[] convertFromIterable(Iterable<S> source) throws ConversionException {
      assert source != null;

      List<T> list = new ArrayList();
      Map<S, T> buffer = new HashMap();
      Iterator var4 = source.iterator();

      while(var4.hasNext()) {
         S sourceElement = var4.next();
         T targetElement = this.convertElement(sourceElement, buffer);
         list.add(targetElement);
      }

      return list.toArray(this.array);
   }

   private T convertElement(S source, Map<S, T> buffer) throws ConversionException {
      if (this.elementConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.elementConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            T target = this.elementConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }

   public interface R {
      R R = (R)Resources.create(R.class);

      @DefaultStringValue("不支持source对象类型“{0}”。")
      String unsupportedSourceClass();
   }
}
